@@ -21,6 +21,7 @@
  *****************************************************************************/
 
 #define MODULE_NAME x11
+#include "modules_inner.h"
 
 /*****************************************************************************
  * Preamble
@@ -38,7 +39,6 @@
 #include "video_output.h"
 
 #include "modules.h"
-#include "modules_inner.h"
 
 /*****************************************************************************
  * Building configuration tree
@@ -51,7 +51,7 @@ MODULE_CONFIG_END
 /*****************************************************************************
  * Capabilities defined in the other files.
  ******************************************************************************/
-extern void vout_getfunctions( function_list_t * p_function_list );
+void _M( vout_getfunctions )( function_list_t * p_function_list );
 
 /*****************************************************************************
  * InitModule: get the module structure and configuration.
@@ -61,7 +61,7 @@ extern void vout_getfunctions( function_list_t * p_function_list );
  * be unloaded later to save memory, and we want to be able to access this
  * data even after the module has been unloaded.
  *****************************************************************************/
-int InitModule( module_t * p_module )
+MODULE_INIT
 {
     p_module->psz_name = MODULE_STRING;
     p_module->psz_longname = "X11 module";
@@ -81,7 +81,7 @@ int InitModule( module_t * p_module )
  * be set to 0 and calls to NeedModule() be made to increment it. To unload
  * the module, one has to wait until i_usage == 0 and call DeactivateModule().
  *****************************************************************************/
-int ActivateModule( module_t * p_module )
+MODULE_ACTIVATE
 {
     p_module->p_functions = malloc( sizeof( module_functions_t ) );
     if( p_module->p_functions == NULL )
@@ -89,7 +89,7 @@ int ActivateModule( module_t * p_module )
         return( -1 );
     }
 
-    vout_getfunctions( &p_module->p_functions->vout );
+    _M( vout_getfunctions )( &p_module->p_functions->vout );
 
     p_module->p_config = p_config;
 
@@ -103,7 +103,7 @@ int ActivateModule( module_t * p_module )
  * returns, i_usage can be set to -1 and the module unloaded. Be careful to
  * lock usage_lock during the whole process.
  *****************************************************************************/
-int DeactivateModule( module_t * p_module )
+MODULE_DEACTIVATE
 {
     free( p_module->p_functions );
 
